@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing
 **
 ** This file is part of Qt Creator.
 **
@@ -9,8 +9,8 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
+** a written agreement between you and The Qt Company.  For licensing terms and
+** conditions see http://www.qt.io/terms-conditions.  For further information
 ** use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
@@ -22,8 +22,8 @@
 ** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** In addition, as a special exception, The Qt Company gives you certain additional
+** rights.  These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ****************************************************************************/
@@ -32,7 +32,8 @@
 #define ACTIONMANAGER_H
 
 #include "coreplugin/core_global.h"
-#include "coreplugin/id.h"
+#include "coreplugin/coreconstants.h"
+#include "coreplugin/icontext.h"
 #include <coreplugin/actionmanager/command.h>
 
 #include <QObject>
@@ -41,15 +42,19 @@
 QT_BEGIN_NAMESPACE
 class QAction;
 class QSettings;
-class QShortcut;
 class QString;
 QT_END_NAMESPACE
 
 namespace Core {
 
 class ActionContainer;
+class Command;
+class Context;
 
-namespace Internal { class MainWindow; }
+namespace Internal {
+class CorePlugin;
+class MainWindow;
+} // Internal
 
 class CORE_EXPORT ActionManager : public QObject
 {
@@ -60,7 +65,9 @@ public:
     static ActionContainer *createMenu(Id id);
     static ActionContainer *createMenuBar(Id id);
 
-    static Command *registerAction(QAction *action, Id id, const Context &context, bool scriptable = false);
+    static Command *registerAction(QAction *action, Id id,
+                                   const Context &context = Context(Constants::C_GLOBAL),
+                                   bool scriptable = false);
 
     static Command *command(Id id);
     static ActionContainer *actionContainer(Id id);
@@ -74,16 +81,16 @@ public:
 
 signals:
     void commandListChanged();
-    void commandAdded(const QString &id);
+    void commandAdded(Core::Id id);
 
 private:
     ActionManager(QObject *parent = 0);
     ~ActionManager();
-    static void initialize();
-    void saveSettings(QSettings *settings);
-    void setContext(const Context &context);
+    static void saveSettings();
+    static void setContext(const Context &context);
 
-    friend class Core::Internal::MainWindow;
+    friend class Core::Internal::CorePlugin; // initialization
+    friend class Core::Internal::MainWindow; // saving settings and setting context
 };
 
 } // namespace Core
